"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Build Forge 
(c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: JavaConversionHelper.py
	
	This contains methods and classes to facilitate converting jython objects to java objects
"""

import java
from java import util
from java import lang
from types import *

import com
from com.ibm.rational.rafw.wsadmin import StringConversionHelper
import types

"""
CLASS :: _JavaConversionHelper 
 This contains methods and classes to facilitate      
 converting jython objects to java objects            
"""
class _JavaConversionHelper:

	"""
	Initialize this object.  
	
	stringFilterMethod is optional and is a method that takes a string and returns a string
		If specified, stringFilterMethod can be used to filter any strings as they are converted to Java
	"""
	def __init__(self, stringFilterMethod=None):
		self.stringFilterMethod = stringFilterMethod
	#endDef
	
	"""
		Convert a sequence to a Java ArrayList.
	
		If the sequence contains dictionaries, and those dictionaries have an id attribute
		then the sequence is sorted by id.
	
		Input: Jython Array []
		Output: ArrayList 
	"""
	def mapSeqToJava (self, seq):
		""" Convert a sequence to a Java ArrayList. """
		result = util.ArrayList(len(seq))
		for e in seq:
			result.add(self.mapToJava(e));
		#endFor
		
		## sort the children if possible
		class _ChildComparator(util.Comparator):
			def compare(self, objOne, objTwo):
				objOneStr = str(objOne)
				if (isinstance(objOne, util.Map)):
					if (objOne.get("id") is not None):
						objOneStr = objOne.get("id")
				objTwoStr = str(objTwo)
				if (isinstance(objTwo, util.Map)):
					if (objTwo.get("id") is not None):
						objTwoStr = objTwo.get("id")
				return (lang.String(objOneStr).compareTo(lang.String(objTwoStr)))
			#endDef
		#endClass
		#sort the list by natural order for compare and external diff
		util.Collections.sort(result, _ChildComparator())
		return result
	#endDef
	
	"""
		Convert a Dictionary to a Java TreeMap. 
	
		Use of TreeMap ensures that attributes are sorted by name.
	
		Input: Jython Dict {}
		Output: TreeMap
	"""
	def mapDictToJava (self, dict):
		# Convert a Dictionary to a Java TreeMap.
		# Using a TreeMap ensures the keys are sorted.
		result = util.TreeMap()
		for key, value in dict.items():
			result.put(self.mapToJava(key), self.mapToJava(value))
		return result
	#endDef
	
	"""
		Convert a Jython type to a Java type. 
		This method includes sub objects, 
		dicts in arrays, arrays in arrays, etc.
	"""
	def mapToJava (self, object):
		""" Convert a Jython type to a Java type. """
		if object is None: return object
		t = type(object)
		if t == TupleType or t == ListType:
			result = self.mapSeqToJava(object)
		elif t == DictType:
			result = self.mapDictToJava(object)
		else:
			if (self.stringFilterMethod is not None):
				result = self.stringFilterMethod(str(object))
			else:
				result = object
		#endIf
		return result
	#endDef
	
	"""
		Takes a string in the form 
		-update -appname Foo -mapModules ['.*', .*, 'Plants']
		into a jython sequence where each element of the list 
		is a parameter or a sub sequence
	"""
	def convertStringToSequence(self, input):
		stringConversion = StringConversionHelper(input)
		optionsAsList = stringConversion.convertToList()
		optionsAsSequence = self._convertToJythonSequence(optionsAsList)
		return optionsAsSequence
	#endDef

	"""
		Recursively converts each List in the supplied javaList into a
		jython sequence.  Each string in the javaList is included in the 
		returned sequence.  Each List in the javaList is converted to 
		a sequence.
	"""
	def _convertToJythonSequence(self, javaList):
		optionsAsSequence = []
		for obj in javaList:
			if ((type(obj) == types.StringType) or (type(obj) == types.UnicodeType)):
				if(obj == StringConversionHelper.EMPTY_ARGUMENT):
					optionsAsSequence.append("")
				else:
					optionsAsSequence.append(str(obj))
			else:
				optionsAsSequence.append(self._convertToJythonSequence(obj))
			#endIf
		#endFor
		return optionsAsSequence
	#endDef	
#endClass

## define global singleton
JavaConversionHelper = _JavaConversionHelper()	
